###################################################
### code chunk number : Loss function
###################################################
## L(x)
## Firstly, set your work directory to contain "main.R".
## Secondly, make sure there is a folder named "figure".
##
rm(list=ls(all=TRUE))
theta = 0.2
n = 10
L_pl_x = function(x) (1/theta - 1)^2 / (1/theta - x) - log(x) - (1/theta - 1)
k1_theta = theta * (1/theta - 1)^2; k1_theta
x = seq(exp(-k1_theta * n), 1/theta * (1 - 1/n), length.out = 1000)
y = L_pl_x(x) 
xp = c(exp(-k1_theta * (n:2)), 1, 1/theta * (1 - 1 / (2:n)))
yp = L_pl_x(xp)
xlim = c(0, 1/theta)
plot(x, y, xlab = expression( x ), ylab = expression( L[pl](x) ), xlim = xlim, type = "l", lwd = 2)
points(xp, yp, pch = c((n-1):1, 0, 1:(n-1)), col = "red", lwd = 2)
abline(h = 0, lty = "dashed")
abline(v = c(0, 1/theta), lty = "dashed")
savePlot(filename = "figure/L[pl](x)", type = c("pdf"))
savePlot(filename = "figure/L[pl](x)", type = c("jpg"))

## L(theta, a)
rm(list=ls(all=TRUE))
theta = 0.2
n = 10
L_pl_a = function(theta, a) theta * (1/theta - 1)^2 / (1 - a) - log(a) + log(theta) - (1/theta - 1)
k2_theta = theta * (1/theta - 1)^2; k2_theta ## k2_theta = k1_theta
a = seq(theta * exp(-k2_theta * n), (1 - 1/n), length.out = 1000)
y = L_pl_a(theta, a) 
ap = c(theta * exp(-k2_theta * (n:2)), theta, 1 - 1 / (2:n))
yp = L_pl_a(theta, ap)
xlim = c(0, 1)
dev.new()
plot(a, y, xlab = expression( a ), ylab = expression( L[pl](paste(a, "|", theta)) ), xlim = xlim, type = "l", lwd = 2)
points(ap, yp, pch = c((n-1):1, 0, 1:(n-1)), col = "red", lwd = 2)
abline(h = 0, lty = "dashed")
abline(v = c(0, 1), lty = "dashed")
savePlot(filename = "figure/L[pl](theta,a)", type = c("pdf"))
savePlot(filename = "figure/L[pl](theta,a)", type = c("jpg"))


###################################################
### code chunk number : Given (alpha, beta, m, n, x)
###################################################
## 
rm(list=ls(all=TRUE))
source("subfunctions.R")
alpha = 2
beta = 1
m = 3
n = 10

set.seed(1)
theta = rbeta(1, shape1 = alpha, shape2 = beta)
x = rbinom(n, size = m, prob = theta); x
res = Compute_PostDist_Estimators_PEPLL(alpha, beta, m, n, x); res

dev.new()
hist(res$PostDist_theta, freq = FALSE, xlab = expression(paste(theta, "|", bold(x))), ylab = expression( pi(paste(theta, "|", bold(x)))),
     main = expression(paste("Histogram of ", theta, "|", bold(x))) ) 
lines(density(res$PostDist_theta), lty = 1, col = c('blue'))
leg.txt = c("Density estimation curve")
legend("topleft", legend = leg.txt, lty = 1, col = c('blue'))
savePlot(filename = "figure/Density_estimation_curve", type = c("pdf"))
savePlot(filename = "figure/Density_estimation_curve", type = c("jpg"))


###################################################
### code chunk number : x change
###################################################
## 
## Given (alpha, beta, m, n), x change
##
rm(list=ls(all=TRUE))
source("subfunctions.R")
alpha = 2
beta = 1
m = 3
n = 10
Seed = 1:10 ## x change
L_x = length(Seed)

E_pl = E_2 = PEPLL_pl = PEPLL_2 = rep(0, L_x)
for (i in 1:L_x){
	set.seed(i)
	theta_i = rbeta(1, shape1 = alpha, shape2 = beta)
	x_i = rbinom(n, size = m, prob = theta_i)
	res_i = Compute_PostDist_Estimators_PEPLL(alpha, beta, m, n, x_i)
	E_pl[i] = res_i$E_pl
	E_2[i] = res_i$E_2
	PEPLL_pl[i] = res_i$PEPLL_pl
	PEPLL_2[i] = res_i$PEPLL_2
}

plot_estimator(E_pl, E_2, parameter = Seed, para_name = "x")
plot_PEPLL(PEPLL_pl, PEPLL_2, parameter = Seed, para_name = "x")


###################################################
### code chunk number : alpha change
###################################################
## 
## Given (beta, m, n), alpha change
##
rm(list=ls(all=TRUE))
source("subfunctions.R")
beta = 1
m = 3
n = 10

alpha = 1:10
L_alpha = length(alpha)

set.seed(1)
E_pl = E_2 = PEPLL_pl = PEPLL_2 = rep(0, L_alpha)
for (i in 1:L_alpha){
	theta_i = rbeta(1, shape1 = alpha[i], shape2 = beta)
	x_i = rbinom(n, size = m, prob = theta_i)
	res_i = Compute_PostDist_Estimators_PEPLL(alpha[i], beta, m, n, x_i)
	E_pl[i] = res_i$E_pl
	E_2[i] = res_i$E_2
	PEPLL_pl[i] = res_i$PEPLL_pl
	PEPLL_2[i] = res_i$PEPLL_2
}

plot_estimator(E_pl, E_2, parameter = alpha, para_name = "alpha")
plot_PEPLL(PEPLL_pl, PEPLL_2, parameter = alpha, para_name = "alpha")


###################################################
### code chunk number : beta change
###################################################
## 
## Given (alpha, m, n), beta change
##
rm(list=ls(all=TRUE))
source("subfunctions.R")
alpha = 2
m = 3
n = 10

beta = 1:10
L_beta = length(beta)

set.seed(1)
E_pl = E_2 = PEPLL_pl = PEPLL_2 = rep(0, L_beta)
for (i in 1:L_beta){
	theta_i = rbeta(1, shape1 = alpha, shape2 = beta[i])
	x_i = rbinom(n, size = m, prob = theta_i)
	res_i = Compute_PostDist_Estimators_PEPLL(alpha, beta[i], m, n, x_i)
	E_pl[i] = res_i$E_pl
	E_2[i] = res_i$E_2
	PEPLL_pl[i] = res_i$PEPLL_pl
	PEPLL_2[i] = res_i$PEPLL_2
}

plot_estimator(E_pl, E_2, parameter = beta, para_name = "beta")
plot_PEPLL(PEPLL_pl, PEPLL_2, parameter = beta, para_name = "beta")


###################################################
### code chunk number : m change
###################################################
## 
## Given (alpha, beta, n), m change
##
rm(list=ls(all=TRUE))
source("subfunctions.R")
alpha = 2
beta = 1
n = 10

m = 1:10
L_m = length(m)

set.seed(1)
E_pl = E_2 = PEPLL_pl = PEPLL_2 = rep(0, L_m)
for (i in 1:L_m){
	theta_i = rbeta(1, shape1 = alpha, shape2 = beta)
	x_i = rbinom(n, size = m[i], prob = theta_i)
	res_i = Compute_PostDist_Estimators_PEPLL(alpha, beta, m[i], n, x_i)
	E_pl[i] = res_i$E_pl
	E_2[i] = res_i$E_2
	PEPLL_pl[i] = res_i$PEPLL_pl
	PEPLL_2[i] = res_i$PEPLL_2
}

plot_estimator(E_pl, E_2, parameter = m, para_name = "m")
plot_PEPLL(PEPLL_pl, PEPLL_2, parameter = m, para_name = "m")


###################################################
### code chunk number : n change
###################################################
## 
## Given (alpha, beta, m), n change
##
rm(list=ls(all=TRUE))
source("subfunctions.R")
alpha = 2
beta = 1
m = 3

n = seq(10, 100, by = 10)
L_n = length(n)

set.seed(1)
E_pl = E_2 = PEPLL_pl = PEPLL_2 = rep(0, L_n)
for (i in 1:L_n){
	theta_i = rbeta(1, shape1 = alpha, shape2 = beta)
	x_i = rbinom(n[i], size = m, prob = theta_i)
	res_i = Compute_PostDist_Estimators_PEPLL(alpha, beta, m, n[i], x_i)
	E_pl[i] = res_i$E_pl
	E_2[i] = res_i$E_2
	PEPLL_pl[i] = res_i$PEPLL_pl
	PEPLL_2[i] = res_i$PEPLL_2
}

plot_estimator(E_pl, E_2, parameter = n, para_name = "n")
plot_PEPLL(PEPLL_pl, PEPLL_2, parameter = n, para_name = "n")


###################################################
### code chunk number : persp3d
###################################################
##
## delta_pl <= delta_2
## PEPLL_pl <= PEPLL_2
## 
rm(list=ls(all=TRUE))
library(rgl) ## Install the R package rgl first.
a = seq(1.1, 10, length = 20); L_a = length(a); L_a
b = seq(0.1, 10, length = 20); L_b = length(b); L_b

E1 = function(a, b){ (b + 1) * b / ((a - 1) * (a + b)) }
E2 = function(a, b){ digamma(a) - digamma(a + b) }
E3 = function(a, b){ (a + b - 1) / (a - 1) }
E4 = function(a, b){ a / (a + b) }

E_pl = function(a, b){ (2 + E1(a, b) - sqrt(E1(a, b) * (E1(a, b) + 4))) / 2 }
E_2 = function(a, b){ E4(a, b) }
PEPLL_pl = function(a, b){ (E1(a, b) + sqrt(E1(a, b) * (E1(a, b) + 4))) / 2 - log(E_pl(a, b)) + E2(a, b) - E3(a, b) + 1 }
PEPLL_2 = function(a, b){ E1(a, b) / (1 - E4(a, b)) - log(E4(a, b)) + E2(a, b) - E3(a, b) + 1 }

E_2_pl = function(a, b){ E_2(a, b) - E_pl(a, b) }
PEPLL_2_pl = function(a, b){ PEPLL_2(a, b) - PEPLL_pl(a, b) }

## Estimators
z_E_2 = outer(a, b, FUN = "E_2")
range(z_E_2)
dim(z_E_2)

z_E_pl = outer(a, b, FUN = "E_pl")
range(z_E_pl)
dim(z_E_pl)

range(z_E_2 - z_E_pl)

z_E_2_pl = outer(a, b, FUN = "E_2_pl")
range(z_E_2_pl)

open3d()
persp3d(a, b, z_E_2,  col = "red", xlab = "a", ylab = "b", zlab = "Estimator")
persp3d(a, b, z_E_pl, col = "blue", add = TRUE)
rgl.postscript("figure/Estimators.pdf", "pdf")

open3d()
persp3d(a, b, z_E_2_pl,  col = "green", xlab = "a", ylab = "b", zlab = "E[2](x) - E[pl](x)")
rgl.postscript("figure/E_2_pl.pdf", "pdf")

## PEPLL
z_PEPLL_2 = outer(a, b, FUN = "PEPLL_2")
range(z_PEPLL_2)
dim(z_PEPLL_2)

z_PEPLL_pl = outer(a, b, FUN = "PEPLL_pl")
range(z_PEPLL_pl)
dim(z_PEPLL_pl)

range(z_PEPLL_2 - z_PEPLL_pl)

z_PEPLL_2_pl = outer(a, b, FUN = "PEPLL_2_pl")
range(z_PEPLL_2_pl)

open3d()
persp3d(a, b, z_PEPLL_2,  col = "red", xlab = "a", ylab = "b", zlab = "PEPLL")
persp3d(a, b, z_PEPLL_pl, col = "blue", add = TRUE)
rgl.postscript("figure/PEPLLs.pdf", "pdf")

open3d()
persp3d(a, b, z_PEPLL_2_pl,  col = "green", xlab = "a", ylab = "b", zlab = "PEPLL[2](x) - PEPLL[pl](x)")
rgl.postscript("figure/PEPLL_2_pl.pdf", "pdf")


###################################################
### code chunk number : A real data example
###################################################
##
rm(list=ls(all=TRUE))
m = 6
N = c(4913, 44, 42, 25, 20, 6, 151)
n = sum(N); n 
x = rep(x = 0:6, times = N) ## n = length(x)

########################
## moment estimator
########################
A1 = mean(x)
M2 = (n - 1) * var(x) / n
alpha1 = (M2 * A1 - A1^2 * (m - A1)) / (A1 * (m - A1) - M2 * m); alpha1
beta1 = (m - A1) * alpha1 / A1; beta1

## Expected Frequency
X = 0:6
Prob1 = choose(m, X) * beta(X + alpha1, m - X + beta1) / beta(alpha1, beta1); Prob1
EF1 = n * Prob1; EF1

source("subfunctions.R")
res1 = Compute_PostDist_Estimators_PEPLL(alpha1, beta1, m, n, x); res1

dev.new()
hist(res1$PostDist_theta, freq = FALSE, xlab = expression(paste(theta, "|", bold(x))), ylab = expression( pi(paste(theta, "|", bold(x)))),
     main = expression(paste("Histogram of ", theta, "|", bold(x))) ) 
lines(density(res1$PostDist_theta), lty = 1, col = c('blue'))
leg.txt = c("Density estimation curve")
legend("topleft", legend = leg.txt, lty = 1, col = c('blue'))

########################
## MLE
########################
source("moment_fun.R"); source("Newtons.R")
p = c(alpha1, beta1); ## The moment estimator is usually a good initial guess.
## p = c(0.01, 0.2) is a good initial guess.
## The MLE is very sensitive to the initial guess. 
res.Newtons = Newtons(fun = moment_fun, x = p); res.Newtons
alpha2 = res.Newtons$root[1]
beta2 = res.Newtons$root[2]

## Expected Frequency
X = 0:6
Prob2 = choose(m, X) * beta(X + alpha2, m - X + beta2) / beta(alpha2, beta2); Prob2
EF2 = n * Prob2; EF2

res2 = Compute_PostDist_Estimators_PEPLL(alpha2, beta2, m, n, x); res2

dev.new()
hist(res2$PostDist_theta, freq = FALSE, xlab = expression(paste(theta, "|", bold(x))), ylab = expression( pi(paste(theta, "|", bold(x)))),
     main = expression(paste("Histogram of ", theta, "|", bold(x))) ) 
lines(density(res2$PostDist_theta), lty = 1, col = c('blue'))
leg.txt = c("Density estimation curve")
legend("topleft", legend = leg.txt, lty = 1, col = c('blue'))

